@@ -25,11 +25,11 @@ from .utils import Thread
 
 OPTIONAL_TEXT = Union[str, bytes, None]
 OPTIONAL_TEXT_OR_ITERABLE = Union[
-    str, bytes, None, List[Union[str, bytes]], Tuple[Union[str, bytes], ...],
+    str, bytes, None, Sequence[Union[str, bytes]],
 ]
 BUFFER = Union[None, io.BytesIO, io.StringIO]
 ARGUMENT = Union[str, Any]
-COMMAND = Union[List[ARGUMENT], Tuple[ARGUMENT, ...], str, Command]
+COMMAND = Union[Sequence[ARGUMENT], str, Command]
 
 
 class PluginInternalError(Exception):
@@ -49,9 +49,7 @@ class FakePopen:
         self,
         command: Union[
             Union[bytes, str],
-            Sequence[
-                Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]
-            ]
+            Sequence[Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]],
         ],
         stdout: OPTIONAL_TEXT_OR_ITERABLE = None,
         stderr: OPTIONAL_TEXT_OR_ITERABLE = None,
@@ -92,6 +90,9 @@ class FakePopen:
                     callable_output, "stderr", self.stderr
                 )
 
+        if self.__thread is not None:
+            self.__thread.join(timeout)
+
         return (
             self.stdout.getvalue() if self.stdout else None,
             self.stderr.getvalue() if self.stderr else None,
@@ -276,7 +277,7 @@ class ProcessDispatcher:
             process_instance.calls.append(command)
 
         if not processes:
-            if not cls._allow_unregistered:
+            if not cls.process_list[-1]._allow_unregistered:
                 raise ProcessNotRegisteredError(
                     "The process '%s' was not registered."
                     % (
@@ -292,7 +293,7 @@ class ProcessDispatcher:
 
         process = processes.popleft()
         if not processes and process_instance is not None:
-            if cls._keep_last_process:
+            if cls.process_list[-1]._keep_last_process:
                 processes.append(process)
             elif command_instance:
                 del process_instance.definitions[command_instance]
@@ -336,14 +337,6 @@ class ProcessDispatcher:
                 return command_instance, processes, process_instance
         return None, None, None
 
-    @classmethod
-    def allow_unregistered(cls, allow: bool) -> None:
-        cls._allow_unregistered = allow
-
-    @classmethod
-    def keep_last_process(cls, keep: bool) -> None:
-        cls._keep_last_process = keep
-
 
 class IncorrectProcessDefinition(Exception):
     """Raised when the register_subprocess() has been called with wrong arguments"""
@@ -359,6 +352,8 @@ class FakeProcess:
             deque
         )
         self.calls: Deque[COMMAND] = deque()
+        self._allow_unregistered: bool = False
+        self._keep_last_process: bool = False
 
     def register_subprocess(
         self,
@@ -429,7 +424,7 @@ class FakeProcess:
     def __exit__(self, *args: List, **kwargs: Dict) -> None:
         ProcessDispatcher.deregister(self)
 
-    def allow_unregistered(cls, allow: bool) -> None:
+    def allow_unregistered(self, allow: bool) -> None:
         """
         Allow / block unregistered processes execution. When allowed, the real
         subprocesses will be called. Blocking will raise the exception.
@@ -437,7 +432,7 @@ class FakeProcess:
         Args:
             allow: decide whether the unregistered process shall be allowed
         """
-        ProcessDispatcher.allow_unregistered(allow)
+        self._allow_unregistered = allow
 
     def call_count(self, command: COMMAND) -> int:
         """
@@ -454,8 +449,7 @@ class FakeProcess:
             command_instance = Command(command)
         return len(tuple(filter(lambda elem: elem == command_instance, self.calls)))
 
-    @classmethod
-    def keep_last_process(cls, keep: bool) -> None:
+    def keep_last_process(self, keep: bool) -> None:
         """
         Keep last process definition from being removed. That can allow / block
         multiple execution of the same command.
@@ -463,7 +457,7 @@ class FakeProcess:
         Args:
             keep: decide whether last process shall be kept
         """
-        ProcessDispatcher.keep_last_process(keep)
+        self._keep_last_process = keep
 
     @classmethod
     def context(cls) -> "FakeProcess":
